//
//  Functions.swift
//  CoreStore
//
//  Copyright (c) 2014 John Rommel Estropia
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in all
//  copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//  SOFTWARE.
//

import Foundation

internal func autoreleasepool<T>(@noescape closure: () -> T) -> T {
    
    var closureValue: T!
    ObjectiveC.autoreleasepool {
        
        closureValue = closure()
    }
    
    return closureValue
}

internal func autoreleasepool<T>(@noescape closure: () throws -> T) throws -> T {
    
    var closureValue: T!
    var closureError: ErrorType?
    ObjectiveC.autoreleasepool {
        
        do {
            
            closureValue = try closure()
        }
        catch {
            
            closureError = error
        }
    }
    
    if let closureError = closureError {
        
        throw closureError
    }
    return closureValue
}

internal func autoreleasepool(@noescape closure: () throws -> Void) throws {
    
    var closureError: ErrorType?
    ObjectiveC.autoreleasepool {
        
        do {
            
            try closure()
        }
        catch {
            
            closureError = error
        }
    }
    
    if let closureError = closureError {
        
        throw closureError
    }
}
